from __future__ import print_function
import datetime
import pickle
import os.path
from googleapiclient.discovery import build
from google_auth_oauthlib.flow import InstalledAppFlow
from google.auth.transport.requests import Request

# If modifying these scopes, delete the file token.pickle.
SCOPES = ['https://www.googleapis.com/auth/calendar.readonly']

class Event:
    def __init__(self, name_, color_, start_, end_):
        self.name = name_
        self.color = color_
        self.start = start_
        self.end = end_

class Calendar:
    def __init__(self, calendarId_, color_):
        self.calendarId = calendarId_
        self.color = color_

def calendarCredentials():
    creds = None
    # The file token.pickle stores the user's access and refresh tokens, and is
    # created automatically when the authorization flow completes for the first
    # time.
    if os.path.exists('token.pickle'):
        with open('token.pickle', 'rb') as token:
            creds = pickle.load(token)
    # If there are no (valid) credentials available, let the user log in.
    if not creds or not creds.valid:
        if creds and creds.expired and creds.refresh_token:
            creds.refresh(Request())
        else:
            flow = InstalledAppFlow.from_client_secrets_file(
                'credentials.json', SCOPES)
            creds = flow.run_local_server(port=0)
        # Save the credentials for the next run
        with open('token.pickle', 'wb') as token:
            pickle.dump(creds, token)

    service = build('calendar', 'v3', credentials=creds)
    
    return service
    
def getCalendarEvents(service, startDate, endDate):
    page_token = None
    calendars = []
    while True:
        calendar_list = service.calendarList().list(pageToken=page_token).execute()
        for calendar_list_entry in calendar_list['items']:
            calendars.append(Calendar(calendar_list_entry['id'], calendar_list_entry['colorId']))
        page_token = calendar_list.get('nextPageToken')
        if not page_token:
            break

    all_events = []

    for calendar in calendars:
        events_result = service.events().list(calendarId=calendar.calendarId, timeMin=startDate,
                                            timeMax=endDate,
                                            maxResults=10, singleEvents=True,
                                            orderBy='startTime').execute()

        for event in events_result.get('items', []):
            
            name = event['summary']
            start = event['start'].get('dateTime')
            end = event['end'].get('dateTime')
            color = event.get('colorId')
            if color == None:
                color = calendar.color

            all_events.append(Event(name, color, start, end))

    return all_events

def main():

    service = calendarCredentials()
    # Call the Calendar API
    now = datetime.datetime.now(datetime.timezone.utc).astimezone()
    now = now.replace(hour=0, minute=0)
    today = now.isoformat() # + '+01:00' # 'Z' indicates UTC time
    print("today: ")
    print(today)
    # one_day = datetime.timedelta(days=1)
    tomorrow = (now.replace(hour=23, minute=0, second=1)).isoformat() # + '+01:00'
    print("tomorrow: ")
    print(tomorrow)
    all_events = getCalendarEvents(service, today, tomorrow)

    # if not events:
    #    print('No upcoming events found.')
    for event in all_events:
        print(event.name + ": " + event.start + ", " + event.color)


if __name__ == '__main__':
    main()
